#!/usr/bin/env python
# encoding: utf-8
"""
workdays.py - calculate the number of workdays between two datetime objects.

Created by Christian Klein on 2006-11-28. Fiddeled with by Maximillian Dornseif.
BSD Licensed.
"""

import datetime
import unittest
from huTools.decorators import memoize

__revision__ = "$Revision: 4288 $"

STATIC_GERMAN_HOLIDAYS = ((1, 1), # Neujahr
                          (5, 1), # Tag der Arbeit
                          (10, 3), # Tag der deutschen Einheit
                          (11, 1), # Allerheiligen
                          (12, 25), # Erster Weihnachtstag
                          (12, 26), # Zweiter Weihnachtstag
)


def add_to_day(day, offset):
    "Returns the date n days before or after day" 
    return datetime.date.fromordinal(day.toordinal() + offset)
    

def easter(year):
    """Returns the day of Easter sunday for 'year'.
    This function only works betweeen 1900 and 2099"""
    h = (24 + 19 * (year % 19)) % 30
    i = h - (h / 28)
    j = (year + (year / 4) + i - 13) % 7
    l = i - j
    
    easter_month = 3 + ((l + 40) / 44)
    easter_day = l + 28 - 31 * (easter_month / 4)
    
    return (easter_month, easter_day)
    

def easter_related_holidays(year):
    "Returns a list of holidays which are related to easter for 'year'."
    easter_days = []
    easter_month, easter_day = easter(year)
    easter_sunday = datetime.date(year, easter_month, easter_day)
    
    # Karfreitag
    easter_days.append(add_to_day(easter_sunday, -2))
    # Ostermontag
    easter_days.append(add_to_day(easter_sunday, 1))
    # Christi Himmelfahrt
    easter_days.append(add_to_day(easter_sunday, 39))
    # Pfingstmontag
    easter_days.append(add_to_day(easter_sunday, 50))
    # Fronleichnam
    easter_days.append(add_to_day(easter_sunday, 60))
    
    return easter_days
    

@memoize
def holidays_german(start, end):
    """Returns a list of dates between start and end that are holidays."""
    hdays = []
    # Berechne alle Feiertage in den Jahren a bis b.
    # Falls a == b werden auch die Feiertage des nächsten
    # Jahres mitberechnet, aber die Liste muss sowieso
    # nochmal gefiltert werden.
    for year in range(start.year, end.year+1):
        for month, day in STATIC_GERMAN_HOLIDAYS:
            hdays.append(datetime.date(year, month, day))
        hdays += easter_related_holidays(year)
    return hdays
    

def workdays(start, end):
    """Calculates the number of working days (Mo-Fr) between two given dates.
    
    Whereas the workdays are calculated siilar to Python slice notation: [start : end[
    Example:
    >>> workdays(datetime.date(2007, 1, 26), datetime.date(2007,  1,  27)) # Fr - Sa
    1
    >>> workdays(datetime.date(2007, 1, 28), datetime.date(2007,  1,  29)) # Su - Mo
    0
    """
    
    def day_after(day):
        """Returns the day after 'day'."""
        return datetime.date.fromordinal(day.toordinal() + 1)
    
    if start > end:
        raise ValueError("can't handle  negative timespan! %r > %r" % (start, end))
    
    # Wenn Anfangstag auf Wochenende liegt, addiere Tage bis Montag
    while start.isoweekday() > 5:
        start = day_after(start)
    
    # Wenn Endtag auf Wochenende liegt, substrahiere Tage bis Freitag
    while end.isoweekday() > 5:
        end = day_after(end)
    
    days = (end - start).days
    
    # Count weekends:
    # if weekday start < weekday end: n / 7
    # if weekday start > weekday end: (n / 7) + 1
    number_of_weekends = days / 7
    if start.isoweekday() > end.isoweekday():
        number_of_weekends += 1
    days = days - 2 * number_of_weekends
    if days < 0:
        raise RuntimeError("%r days difference %r|%r|%r" % (days, start, end, number_of_weekends))
    return days
    

@memoize
def workdays_german(start, end):
    """Calculates the number of working days between two given dates while considering german holidays."""    
    days = workdays(start, end)
    # Deduct Holidays (but only the ones not on weekends)
    holid = [x for x in holidays_german(start, end) if (x >= start) and (x < end) and (x.isoweekday() < 6)]
    return days - len(holid)
    

def workdayhours_german(start, end):
    """Calculates the number of hours expect weekends and german holidays between two given datetimes."""
    noncountingdays = (end.date() - start.date()) - \
                       datetime.timedelta(workdays_german(start.date(), end.date()))
    delta = (end - start - noncountingdays)
    return (delta.days * 24) + (delta.seconds / 60.0 / 60.0)
    

def is_workday_german(day):
    """Checks if a day is a workday in germany (NRW).
    
    >>> is_workday_german(datetime.date(2007, 1, 1))
    False
    >>> is_workday_german(datetime.date(2007, 1, 2))
    True
    """
    if day.isoweekday() > 5:
        return False # weekend
    if day in holidays_german(day, day):
        return False
    return True
    

@memoize
def next_workday_german(startday):
    """Returns the next workday after startday.
    
    >>> next_workday_german(datetime.date(2006, 12, 29))
    datetime.date(2007, 1, 2)
    """
    
    day_ordinal = startday.toordinal() + 1
    while not is_workday_german(datetime.date.fromordinal(day_ordinal)):
        day_ordinal += 1
    return datetime.date.fromordinal(day_ordinal)
    

@memoize
def previous_workday_german(startday):
    """Returns the workday before startday.
    
    >>> previous_workday_german(datetime.date(2007, 1, 2))
    datetime.date(2006, 12, 29)
    """
    
    day_ordinal = startday.toordinal() - 1
    while not is_workday_german(datetime.date.fromordinal(day_ordinal)):
        day_ordinal -= 1
    return datetime.date.fromordinal(day_ordinal)
    

def add_workdays_german(startday, count):
    """Adds <count> workdays to <startday>."""
    
    day = startday
    while count > 0:
        day = next_workday_german(day)
        count -= 1
    while count < 0:
        day = previous_workday_german(day)
        count += 1
    return day
    

class _WorkdayTests(unittest.TestCase):
    """Testcases for workdays module. Calendar hint:
        November 2006         December 2006          January 2007 
     S  M Tu  W Th  F  S   S  M Tu  W Th  F  S   S  M Tu  W Th  F  S
              1  2  3  4                  1  2      1  2  3  4  5  6
     5  6  7  8  9 10 11   3  4  5  6  7  8  9   7  8  9 10 11 12 13
    12 13 14 15 16 17 18  10 11 12 13 14 15 16  14 15 16 17 18 19 20
    19 20 21 22 23 24 25  17 18 19 20 21 22 23  21 22 23 24 25 26 27
    26 27 28 29 30        24 25 26 27 28 29 30  28 29 30 31         
                          31
    """
    
    def test_workdays(self):
        """Simple minded tests for workdays()."""
        date = datetime.date
        
        self.assertEqual(0, workdays(date(2007, 1, 25), date(2007, 1, 25))) # Th - Th
        self.assertEqual(1, workdays(date(2007, 1, 25), date(2007, 1, 26))) # Th - Fr
        self.assertEqual(2, workdays(date(2007, 1, 25), date(2007, 1, 27))) # Th - Sa
        self.assertEqual(1, workdays(date(2007, 1, 26), date(2007, 1, 27))) # Fr - Sa
        self.assertEqual(1, workdays(date(2007, 1, 26), date(2007, 1, 28))) # Fr - Su
        self.assertEqual(1, workdays(date(2007, 1, 26), date(2007, 1, 29))) # Fr - Mo
        self.assertEqual(0, workdays(date(2007, 1, 28), date(2007, 1, 29))) # Su - Mo
        self.assertEqual(2, workdays(date(2007, 1, 26), date(2007, 1, 30))) # Fr - Tu
        self.assertEqual(1, workdays(date(2007, 1, 28), date(2007, 1, 30))) # Su - Tu
                                                                      
        self.assertEqual(0, workdays(date(2007, 1, 26), date(2007, 1, 26))) # Fr - Fr
        self.assertEqual(1, workdays(date(2007, 1, 26), date(2007, 1, 27))) # Fr - Sa
        self.assertEqual(1, workdays(date(2007, 1, 26), date(2007, 1, 28))) # Fr - Su
        self.assertEqual(0, workdays(date(2007, 1, 27), date(2007, 1, 28))) # Sa - So
        self.assertEqual(0, workdays(date(2007, 1, 27), date(2007, 1, 29))) # Sa - Mo
        self.assertEqual(1, workdays(date(2007, 1, 27), date(2007, 1, 30))) # Fr - Tu
                            
        self.assertEqual(0, workdays(date(2006, 11, 29), date(2006, 11, 29))) # We - We
        self.assertEqual(1, workdays(date(2006, 11, 29), date(2006, 11, 30))) # We - Th
        self.assertEqual(1, workdays(date(2006, 11, 30), date(2006, 12, 1))) # Th - Fr
        self.assertEqual(5, workdays(date(2006, 12, 12), date(2006, 12, 19))) # Tu - Tu
        self.assertEqual(0, workdays(date(2006, 11, 20), date(2006, 11, 20)))
        self.assertEqual(1, workdays(date(2006, 11, 20), date(2006, 11, 21)))
        self.assertEqual(4, workdays(date(2006, 11, 20), date(2006, 11, 24)))
        self.assertEqual(5, workdays(date(2006, 11, 20), date(2006, 11, 25))) # Mo - Sa
        self.assertEqual(5, workdays(date(2006, 11, 20), date(2006, 11, 26))) # Mo - Su
        self.assertEqual(5, workdays(date(2006, 11, 20), date(2006, 11, 27)))
        self.assertEqual(5, workdays(date(2006, 12, 25), date(2007, 1, 1)))
        self.assertEqual(6, workdays(date(2006, 12, 8), date(2006, 12, 18)))
        self.assertEqual(5, workdays(date(2006, 12, 9), date(2006, 12, 18)))
        self.assertEqual(0, workdays(date(2006, 12, 17), date(2006, 12, 18)))
        self.assertEqual(1, workdays(date(2006, 12, 17), date(2006, 12, 19)))
        self.assertEqual(4, workdays(date(2006, 12, 17), date(2006, 12, 22)))
        self.assertEqual(5, workdays(date(2006, 12, 17), date(2006, 12, 23))) # Su - Sa
        self.assertEqual(5, workdays(date(2006, 12, 17), date(2006, 12, 24))) # Su - Su
        self.assertEqual(5, workdays(date(2006, 12, 17), date(2006, 12, 25)))
        self.assertEqual(261, workdays(date(2004, 1, 1), date(2004, 12, 31)))
        self.assertEqual(260, workdays(date(2005, 1, 1), date(2005, 12, 31)))
        self.assertEqual(260, workdays(date(2006, 1, 1), date(2006, 12, 31)))
        self.assertEqual(260, workdays(date(2007, 1, 1), date(2007, 12, 31)))
        self.assertEqual(261, workdays(date(2008, 1, 1), date(2008, 12, 31)))
        self.assertEqual(260+260, workdays(date(2005, 1, 1), date(2006, 12, 31)))
        self.assertEqual(260+260+260, workdays(date(2005, 1, 1), date(2007, 12, 31)))
        # TODO: this fails! leap year issue?
        # self.assertEqual(261+260, workdays(date(2004, 1, 1), date(2005,  12,  31)))
        # self.assertEqual(261+260+260, workdays(date(2004, 1, 1), date(2006,  12,  31)))
        # self.assertEqual(260+260+260+261, workdays(date(2005, 1, 1), date(2008,  12,  31)))
    
    def test_workdays_german(self):
        """Simple minded tests for workdays_german()."""
        date = datetime.date
        self.assertEqual(0, workdays_german(date(2007, 1, 25), date(2007, 1, 25))) # Th - Th
        self.assertEqual(1, workdays_german(date(2007, 1, 25), date(2007, 1, 26))) # Th - Fr
        self.assertEqual(2, workdays_german(date(2007, 1, 25), date(2007, 1, 27))) # Th - Sa
        self.assertEqual(1, workdays_german(date(2007, 1, 26), date(2007, 1, 27))) # Fr - Sa
        self.assertEqual(1, workdays_german(date(2007, 1, 26), date(2007, 1, 28))) # Fr - Su
        self.assertEqual(1, workdays_german(date(2007, 1, 26), date(2007, 1, 29))) # Fr - Mo
        self.assertEqual(0, workdays_german(date(2007, 1, 28), date(2007, 1, 29))) # Su - Mo
        self.assertEqual(2, workdays_german(date(2007, 1, 26), date(2007, 1, 30))) # Fr - Tu
        self.assertEqual(1, workdays_german(date(2007, 1, 28), date(2007, 1, 30))) # Su - Tu
                                                                             
        self.assertEqual(0, workdays_german(date(2007, 1, 26), date(2007, 1, 26))) # Fr - Fr
        self.assertEqual(1, workdays_german(date(2007, 1, 26), date(2007, 1, 27))) # Fr - Sa
        self.assertEqual(1, workdays_german(date(2007, 1, 26), date(2007, 1, 28))) # Fr - Su
        self.assertEqual(0, workdays_german(date(2007, 1, 27), date(2007, 1, 28))) # Sa - So
        self.assertEqual(0, workdays_german(date(2007, 1, 27), date(2007, 1, 29))) # Sa - Mo
        self.assertEqual(1, workdays_german(date(2007, 1, 27), date(2007, 1, 30))) # Fr - Tu
        
        self.assertEqual(3, workdays_german(date(2006, 12, 25), date(2007, 1, 1)))
        self.assertEqual(1, workdays_german(date(2007, 2, 2), date(2007, 2, 5)))
        self.assertEqual(252, workdays_german(date(2005, 1, 1), date(2005, 12, 31)))
        self.assertEqual(250, workdays_german(date(2006, 1, 1), date(2006, 12, 31)))
        self.assertEqual(249, workdays_german(date(2007, 1, 1), date(2007, 12, 31)))
        self.assertEqual(251, workdays_german(date(2008, 1, 1), date(2008, 12, 31)))
        # Christi Himmelfahrt
        self.assertEqual(1, workdays_german(date(2007, 5, 16), date(2007, 5, 17)))
        self.assertEqual(1, workdays_german(date(2007, 5, 16), date(2007, 5, 18)))
        # Pfingsten         
        self.assertEqual(1, workdays_german(date(2007, 5, 24), date(2007, 5, 25))) # Th - Fr
        self.assertEqual(1, workdays_german(date(2007, 5, 25), date(2007, 5, 26))) # Fr - Sa
        self.assertEqual(1, workdays_german(date(2007, 5, 25), date(2007, 5, 27))) # Fr - So
        self.assertEqual(1, workdays_german(date(2007, 5, 25), date(2007, 5, 28))) # Fr - Mo
        self.assertEqual(1, workdays_german(date(2007, 5, 25), date(2007, 5, 29))) # Fr - Tu
        # Christi Himmelfahrt
        self.assertEqual(1, workdays_german(date(2007, 6, 6), date(2007, 6, 7)))
        self.assertEqual(1, workdays_german(date(2007, 6, 6), date(2007, 6, 8)))
        self.assertEqual(252+250, workdays_german(date(2005, 1, 1), date(2006, 12, 31)))
        self.assertEqual(252+250+249, workdays_german(date(2005, 1, 1), date(2007, 12, 31)))
        # TODO: this fails! leap year issue?
        # self.assertEqual(252+250+249+251, workdays_german(date(2005, 1, 1), date(2008,  12,  31)))
        
    def test_workdayhours_german(self):
        """Simple minded tests for workdays_german()."""
        date = datetime.datetime
        self.assertEqual(72, workdayhours_german(date(2006, 12, 25), date(2007, 1, 1)))
        self.assertEqual(24, workdayhours_german(date(2007, 2, 2), date(2007, 2, 5)))
        self.assertAlmostEqual(30.2, workdayhours_german(date(2007, 2, 2, 10, 47),
                                                         date(2007, 2, 5, 16, 59)))
        
    def test_next_workday_german(self):
        """Simple minded tests for next_workday_german()."""
        date = datetime.date
        self.assertEqual(date(2007, 5, 21), next_workday_german(date(2007, 5, 18))) # Fr
        self.assertEqual(date(2007, 5, 22), next_workday_german(date(2007, 5, 21))) # Mo
        self.assertEqual(date(2007, 5, 25), next_workday_german(date(2007, 5, 24))) # Th
        # Pfingsten
        self.assertEqual(date(2007, 5, 29), next_workday_german(date(2007, 5, 25))) # Fr
        self.assertEqual(date(2007, 5, 29), next_workday_german(date(2007, 5, 26))) # Sa
        self.assertEqual(date(2007, 5, 29), next_workday_german(date(2007, 5, 27))) # Su
        self.assertEqual(date(2007, 5, 29), next_workday_german(date(2007, 5, 28))) # Mo ( Holiday)
        self.assertEqual(date(2007, 5, 31), next_workday_german(date(2007, 5, 30))) 
    
    def test_add_workdays_german(self):
        """Simple minded tests for add_workdays_german,"""
        date = datetime.date
        self.assertEqual(date(2008, 11, 24), add_workdays_german(date(2008, 11, 24), 0)) 
        self.assertEqual(date(2008, 11, 25), add_workdays_german(date(2008, 11, 24), 1)) 
        self.assertEqual(date(2008, 12, 1), add_workdays_german(date(2008, 11, 24), 5)) 
        self.assertEqual(date(2008, 11, 21), add_workdays_german(date(2008, 11, 24), -1)) 
        self.assertEqual(date(2008, 11, 14), add_workdays_german(date(2008, 11, 24), -6)) 
        
    
if __name__ == '__main__':
    import doctest
    doctest.testmod()
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
formats.py - formating and parsing of timestamps.

Created by Maximillian Dornseif on 2007-06-24.
Copyright (c) 2007 HUDORA GmbH. All rights reserved.
"""

import unittest
import datetime
import time
import email.utils

__revision__ = "$Revision: 3687 $"


def rfc3339_date(date):
    """Formates a datetime object according to RfC 3339."""
    return date.strftime('%Y-%m-%dT%H:%M:%SZ')
    

def rfc3339_date_parse(date):
    """Parses an RfC 3339 timestamp into a datetime object."""
    return datetime.datetime.strptime(date, '%Y-%m-%dT%H:%M:%SZ')
    

# RfC 2616 is a subset of RFC 1123 date
# Weekday and month names for HTTP date/time formatting; always English!


def rfc2616_date(date):
    """Formates a datetime object according to RfC 2616."""
    return email.utils.formatdate(time.mktime(date.timetuple()), usegmt=True)
    

def rfc2616_date_parse(data):
    """Parses an RfC 2616/2822 timestapm into a datetime object."""
    return datetime.datetime.fromtimestamp(email.utils.mktime_tz(email.utils.parsedate_tz(data)))
    

class _FormatsTests(unittest.TestCase):
    # TODO: investigate handling of timezones.
    
    def test_rfc3339_date(self):
        """Test basic rfc3339_date output."""
        self.assertEqual(rfc3339_date(datetime.datetime(2007, 2, 3, 4, 5, 6)), '2007-02-03T04:05:06Z')
    
    def test_rfc3339_date_parse(self):
        """Test basic rfc3339_date_parse output."""
        self.assertEqual(rfc3339_date_parse('2007-02-03T04:05:06Z'),
                         datetime.datetime(2007, 2, 3, 4, 5, 6))
    
    def test_rfc2616_date(self):
        """Test basic rfc2616_date output."""
        self.assertEqual(rfc2616_date(datetime.datetime(2007, 2, 3, 4, 5, 6)),
                         'Sat, 03 Feb 2007 03:05:06 GMT')
    
    def test_rfc2616_date_parse(self):
        """Test basic rfc2616_date_parse output."""
        self.assertEqual(rfc2616_date_parse('Sat, 03 Feb 2007 03:05:06 GMT'),
                         datetime.datetime(2007, 2, 3, 4, 5, 6))
    

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
luids.py

Tools for generating various local unique IDs.

This module tries to generate IDs which are guaranteed to be unique on a machine. Compared with the fine uuid
module in python 2.5 we try to be unique only on a single host and not globally. In turn we provide stronger
guarantees that no ID will be ever duplicate on a machine. Therfor our IDs are perfect for generating unique
filenames in a directory used by severall processes/threads.

See http://docs.python.org/lib/module-uuid.html and http://zesty.ca/python/uuid.py for the Python standard
uuid module.

Created by Maximillian Dornseif on 2006-11-08. BSD Licensed.
"""

import os
import time
import socket
import unittest

try:
    import thread as _thread
except ImportError:
    import dummy_thread as _thread

__revision__ = "$Revision: 3687 $"

_counter = 0
_counter_lock = _thread.allocate_lock()


def unique_machine32():
    """
    Generate an ID which has a low probability of repeating on this machine.
    
    What goes where:
    
      PP ^
    TTtt ^
    c
    
    So the first byte is an combination of a counter (c) and the first byte of the least significant 
    (and thus fastest changing) word (TT) of the current timestamp. This is followed by the next byte of the 
    least significant word of the current timestamp (TT). The next two bytes are the PID (PP) and the most 
    significant word of the current timestamp (tt) XORed.
    
    Assuming 16bit PIDs this should result in unique IDs on a machine even with multi-threading. But there 
    are degenerated scenarios (more than 2**16 concurrent/fast spawning processes, while there are more than 
    2**8 threads bussy generating unique IDs) where the IDs might not be unique.
    To avoid these use unique_machine64.
    """

    global _counter
    try: # Entering critical section.
        _counter_lock.acquire()
        intnow = long(time.time())
        _counter = (_counter + 1) % 0xff
        # shift more significant bytes of time up so they don't overlap PID
        ret = (os.getpid() ^ (intnow << 16) ^ (intnow >> 16) ^ (_counter << 24)) % 0xffffffff
    finally:
        _counter_lock.release()
    return ret
    

def unique_machine64():
    """
    Generate an ID which should never repeat on this machine.
    
    This function is suggested as the base for generating filenames and the like. Processes/Threads 
    running on the same machine should never be able to generate the same ID and no ID should be created 
    twice.
    
    What goes where:
    TTTT     ^
        PP   ^
        cccc
        
    The first two words are the current timestamp. The next word is the current  PID xored with the most
    significant word of a counter. The last byte is the most significant word of a counter. 
    The counter is being increased by 1 in every call. This should make this function thread 
    save unless you call it more than 2**16 times per second.
    """
    
    global _counter
    try: # Entering critical section.
        _counter_lock.acquire()
        intnow = long(time.time())
        _counter = (_counter + 1) % 0xffffffff
        ret = ((intnow << 32) ^ (os.getpid() << 16) ^ (_counter)) % 0xffffffffffff
    finally:
        _counter_lock.release()
    return ret
    

_hostname = None


def luid():
    """Generate an ID which should be globally unique.
    
    This only works if gethostname() returns an unique name. This usually is true to a certain degree if you 
    use your machine in an NIS/NFS configuration. So this function is mainly usefull to generate unique
    filenames for NFS-shared filesystems.
    
    So this is very well suited for generationg IDs in a setup where you controll all the machines and thus
    hostnames."""
    
    global _hostname
    if not _hostname:
        _hostname = socket.gethostname()
    return "%s%x" % (_hostname, unique_machine64())


# Testcases


class _uuidsTests(unittest.TestCase):
    """Simple Testcases for uuid generation."""
    
    def test_unique_machine32(self):
        """Basic tests for unique_machine32."""
        seen = set()
        for i in range(250):
            uuid = unique_machine32()
            self.assertTrue((uuid <= 0xffffffff) and (uuid >= 0))
            self.assertFalse(uuid in seen)
            seen.add(uuid)
        time.sleep(1) # avois wrap arround of counter
        for i in range(250):
            uuid = unique_machine32()
            self.assertTrue((uuid <= 0xffffffff) and (uuid >= 0))
            self.assertFalse(uuid in seen)
            seen.add(uuid)
    
    def test_unique_machine64(self):
        """Basic tests for unique_machine64."""
        seen = set()
        for i in range(30000):
            uuid = unique_machine64()
            self.assertFalse(uuid in seen)
            self.assertTrue((uuid <= 0xffffffffffff) and (uuid >= 0))
            seen.add(uuid)
    
if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
decorators.py

Created by Maximillian Dornseif on 2007-05-10.
Copyright (c) 2007 HUDORA GmbH. All rights reserved.
"""


from decorator import decorator


def _getattr_(obj, name, default_thunk):
    "Similar to .setdefault in dictionaries."
    try:
        return getattr(obj, name)
    except AttributeError:
        default = default_thunk()
        setattr(obj, name, default)
        return default

# from http://www.phyast.pitt.edu/~micheles/python/documentation.html
@decorator
def memoize(func, *args):
    """This decorator caches the results of the function it decorates.
    
    See http://en.wikipedia.org/wiki/Memoisation for fare more than ypou ever want to know on that topic.
    """
    
    dic = _getattr_(func, "memoize_dic", dict)
    # memoize_dic is created at the first call
    if args in dic:
        return dic[args]
    else:
        result = func(*args)
        dic[args] = result
        return result


# from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/425445
def func_once(func):
    "A decorator that runs a function only once."
    def decorated(*args, **kwargs):
        try:
            return decorated._once_result
        except AttributeError:
            decorated._once_result = func(*args, **kwargs)
            return decorated._once_result
    return decorated


def method_once(method):
    "A decorator that runs a method only once."
    attrname = "_%s_once_result" % id(method)
    def decorated(self, *args, **kwargs):
        try:
            return getattr(self, attrname)
        except AttributeError:
            setattr(self, attrname, method(self, *args, **kwargs))
            return getattr(self, attrname)
    return decorated



#!/usr/bin/env python
# encoding: utf-8
"""
world.py - countries of the world (to be procise: countries we do buiseness with)

Created by Maximillian Dornseif on 2007-05-01.
Copyright (c) 2007 HUDORA GmbH. BSD Licensed.
"""

__revision__ = "$Revision: 3951 $"

COUNTRY_CHOICES = [('DE', 'Deutschland')] \
                   + sorted([
                   ('AT', 'Östereich'),
                   ('CH', 'Schweiz'),
                   ('BE', 'Belgien'),
                   ('FR', 'Frankreich'),
                   ('ES', 'Spanien'),
                   ('NL', 'Niederlande'),
                   ('IT', 'Italien'),
                   ('GB', 'Grossbritannien'),
                   ('LI', 'Lichtenstein'),
                   ('LU', 'Luxenburg'),
                   ('PT', 'Portugal'),
                   ('CZ', 'Tschechien'),
                   ('SE', 'Schweden'),
                   ('GR', 'Griechenland'),
                   ('DK', 'Daenemark'),
                   ('US', 'USA'),
                   ('AR', 'Argentinien'),
                   ('HU', 'Ungarn'),
                   ('LV', 'Lettland'),
                   ('PL', 'Polen'),
                   ('LT', 'Littauen'),
                   ('SK', 'Slowakei'),
                   ('HR', 'Kroatioen'),
                   ('IE', 'Irland'),
                   ('SI', 'Slowenien'),
                   ('EE', 'Estland'),
                   ('BG', 'Bulgarien'),
                   ('NO', 'Norwegen'),
                   ('AU', 'Australien'),
                  ])



# encoding: utf-8
"""Alternative implementation of workdays. kept for testing purposes."""

from datetime import timedelta, datetime

__revision__ = "$Revision: 3687 $"


def calculate_weekenddays_between(datetime1, datetime2):
    """
        Returns the number of weekend days between the 2 dates passed as parameters.
        The result is returned in form of a positive timedelta object. This means that
        it doesn't matter the order of the datetimes given as parameters.
    """
    if datetime1 == datetime2:
        return timedelta(0)
    ## The dates are ordered so d1 < d2
    if datetime1 < datetime2:
        d1 = datetime1
        d2 = datetime2
    else:
        d1 = datetime2
        d2 = datetime1

        ## monday1 = the monday after d1
    monday1 = (d1 + timedelta(8 - d1.isoweekday())).replace(hour = 0, minute = 0, second = 0, microsecond = 0)
    ## monday2 = the monday before d2
    monday2 = (d2 - timedelta(d2.isoweekday() - 1)).replace(hour = 0, minute = 0, second = 0, microsecond = 0)

    ## if monday1 <= monday2 => the dates are not in the same week
    if monday1 <= monday2:
        dif1 = monday1 - d1
        if dif1.days > 2:
            dif1 = timedelta(2)
        dif2 = d2 - monday2
        if dif2.days <= 5:
            dif2 = timedelta()
        else:
            dif2 = dif2 - timedelta(5)
        return dif1 + dif2 + (monday2 - monday1) / 7 * 2
    ## the dates are in the same week
    else:
        ## both dates are working days
        if d2.isoweekday() <= 5:
            return timedelta()
        ## both dates are weekend days
        elif d1.isoweekday() > 5:
            return d2 - d1
        else:
            return d2 - monday2 - timedelta(5)
    

def calculate_workingdays_between(datetime1, datetime2):
    """
        Returns the number of workingdays days between the 2 dates passed as parameters.
        The result is returned in form of a positive timedelta object. This means that
        it doesn't matter the order of the datetimes given as parameters.
    """
    if datetime1 == datetime2:
        return timedelta(0)
    ## The dates are ordered so d1 < d2
    if datetime1 < datetime2:
        d1 = datetime1
        d2 = datetime2
    else:
        d1 = datetime2
        d2 = datetime1
    return d2 - d1 - calculate_weekenddays_between(d1, d2)
    

def main():
    """
    Testing the functions
    """
    ## 2 dates in the same week
    dt1 = datetime(2006, 11, 29)    # working day
    dt2 = datetime(2006, 11, 27)    # working day
    print dt1
    print dt2
    print "weekenddays: " + str(calculate_weekenddays_between(dt1, dt2))
    print "workingdays: " + str(calculate_workingdays_between(dt1, dt2))

    dt1 = datetime(2006, 11, 29)    # working day
    dt2 = datetime(2006, 12, 2)     # saturday
    print dt1
    print dt2
    print "weekenddays: " + str(calculate_weekenddays_between(dt1, dt2))
    print "workingdays: " + str(calculate_workingdays_between(dt1, dt2))

    dt1 = datetime(2006, 11, 3)    # sunday
    dt2 = datetime(2006, 12, 2)    # saturday
    print dt1
    print dt2
    print "weekenddays: " + str(calculate_weekenddays_between(dt1, dt2))
    print "workingdays: " + str(calculate_workingdays_between(dt1, dt2))

    ## 2 dates in consecutive weeks
    dt1 = datetime(2006, 11, 29)    # working day
    dt2 = datetime(2006, 12, 5)     # working day
    print dt1
    print dt2
    print "weekenddays: " + str(calculate_weekenddays_between(dt1, dt2))
    print "workingdays: " + str(calculate_workingdays_between(dt1, dt2))

    dt1 = datetime(2006, 11, 29)    # working day
    dt2 = datetime(2006, 12, 10)     # working day
    print dt1
    print dt2
    print "weekenddays: " + str(calculate_weekenddays_between(dt1, dt2))
    print "workingdays: " + str(calculate_workingdays_between(dt1, dt2))

    ## 2 dates in different weeks
    dt1 = datetime(2006, 11, 26)    # sunday
    dt2 = datetime(2006, 12, 5)     # working day
    print dt1
    print dt2
    print "weekenddays: " + str(calculate_weekenddays_between(dt1, dt2))
    print "workingdays: " + str(calculate_workingdays_between(dt1, dt2))

    dt1 = datetime(2006, 11, 24)    # working day
    dt2 = datetime(2006, 12, 6)     # working day
    print dt1
    print dt2
    print "weekenddays: " + str(calculate_weekenddays_between(dt1, dt2))
    print "workingdays: " + str(calculate_workingdays_between(dt1, dt2))

if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
Credit System for Python Code

This program analyzes Python code with pylint
and calculates a credit for the code, depending
on a miimum required score and the previous run.
"""

import sys
from pylint.lint import PyLinter
from pylint import checkers, config
import sqlobject

DATABASE_URI = "sqlite:///Users/chris/Desktop/database.db"
MINIMUM_SCORE = 8.0


class PythonScore(sqlobject.SQLObject):
    """
    OO mapping of the score for a Python file
    """
    username = sqlobject.StringCol()
    pathname = sqlobject.StringCol()
    revision = sqlobject.StringCol()
    score = sqlobject.FloatCol()
    old_score = sqlobject.FloatCol()
    credit = sqlobject.FloatCol()
    date = sqlobject.DateTimeCol(default=sqlobject.DateTimeCol.now)


def process_file(filename):
    """
    Analyze the file with pylint and write the result
    to a database
    """
    linter = PyLinter()

    checkers.initialize(linter)
    linter.read_config_file()
    linter.quiet = 1
    
    filemods = linter.expand_files((filename, ))
    if filemods:
        old_stats = config.load_results(filemods[0].get('basename'))
        old_score = old_stats.get('global_note', 0.0)
    
    linter.check(filename)
    score = eval(linter.config.evaluation, {}, linter.stats)
    
    # Calculate the credit for both scores
    if score < 0:
        credit = 2.0 * score
    elif score < old_score:
        credit = -1.5 * (old_score - score)
    elif score < MINIMUM_SCORE:
        credit = -1.5 * (MINIMUM_SCORE - score)
    else:
        credit = score - old_score
    
    return score, old_score, credit
    

def main(repos, revision):
    """
    Main function.
    """

    import pysvn
    import os.path
    
    client = pysvn.Client()
    diff = client.diff_summarize(repos,
             revision1=pysvn.Revision(pysvn.opt_revision_kind.number, revision-1),
             revision2=pysvn.Revision(pysvn.opt_revision_kind.number, revision))

    conn = sqlobject.connectionForURI(DATABASE_URI)
    sqlobject.sqlhub.processConnection = conn
    #PythonScore.createTable()

    func = lambda f: os.path.splitext(f.path)[-1] == ".py"
    for entry in filter(func, diff):
        path = os.path.join(repos, entry.path)
        score, old_score, credit = process_file(path)
        
        info = client.info(path)
        
        PythonScore(username=info['commit_author'], pathname=path, revision="1",
                score=score, old_score=old_score, credit=credit)
    

if __name__ == "__main__":
    # We are probably called as a subversion post-commit hook
    if len(sys.argv) <= 2:
        sys.stderr.write("Usage: %s repository revision\n" % (sys.argv[0]))
        sys.exit()

    main(sys.argv[1], int(sys.argv[2]))



#!/usr/bin/env python
# encoding: utf-8
"""ReReadingConfigParser is designed to be mainly compatible with
ConfigParser but checks before access to values if the
configuration file on disk has changed and re-reads the file if
necessary.

The detection of filechange is done by comparing the mtime of the
file with the mtime the file had when it was last read.

A big difference to the original ConfigParser is that
ReReadingConfigParser does not support setting Values or writing the
configuration to disk.

ReReadingConfigParser.read() only work with a single file not with
lists of filenames. ReReadingConfigParser.readfp() only works with
file like objects which have a name associated and can be
os.stat()ed.

add_section(), set(), write(), remove_option() and
remove_section() are not supported.


Authors:

This Software was written by md@hudora.de for the twisd AG,
Bonn, Germany. The twisd AG kindly donated it as Freie Software.
"""

__version__ = '$Id: ReReadingConfigParser.py 3687 2008-08-16 08:29:51Z m.dornseif@hudora.de $'
  
__copyright__ = """(c) 2001 twisd AG, Bonn - http://www.twisd.de/
further distribution is granted under the terms of LGPL or classical
MIT Licence."""

__revision__ = "$Revision: 3687 $"

from ConfigParser import ConfigParser
import os
import stat


class ReReadingConfigParser(ConfigParser):

    def ReReadIfChanged(self):
        mtime = os.stat(self.config_name)[stat.ST_MTIME]
        if mtime > self.config_mtime:
            self.read(self.config_name) 
            
    def read(self, fpname):
        # try to find out last modified date of file
        self.config_mtime = os.stat(fpname)[stat.ST_MTIME]
        self.config_name = fpname 
        return ConfigParser.read(self, fpname)

    def readfp(self, fdescriptor, filename=None):
        if filename is None:
            filename = fdescriptor.name
            ConfigParser.read(self, filename)

    def sections(self):
        self.ReReadIfChanged()
        return ConfigParser.sections(self)

    def options(self, section):
        self.ReReadIfChanged()
        return ConfigParser.options(self, section)
    
    def get(self, section, option, raw=0, myvars=None):
        self.ReReadIfChanged()
        return ConfigParser.get(self, section, option, raw, myvars)
    
    # ReReadingConfigParser is read only, so overwrite configuration
    # changing commands
    
    def add_section(self, section):
        raise NotImplementedError
    
    def set(self, section, option, value):
        raise NotImplementedError

    def write(self, fdescriptor):
        raise NotImplementedError

    def remove_option(self, section, option):
        raise NotImplementedError

    def remove_section(self, section):
        raise NotImplementedError
 

def test():
    import time
    
    cparser = ReReadingConfigParser()
    fdescriptor = open('ReReadingConfigParser.TESTFILE', 'w')
    fdescriptor.write('[TESTSECTION]\nvalue: 1\n')
    fdescriptor.close()
    
    cparser.read('ReReadingConfigParser.TESTFILE')
    assert cparser.get('TESTSECTION', 'value') == '1'

    # to be sure mtime has changed
    time.sleep(2)
    fdescriptor = open('ReReadingConfigParser.TESTFILE', 'w')
    fdescriptor.write('[TESTSECTION]\nvalue: 2\n')
    fdescriptor.close()

    assert cparser.get('TESTSECTION', 'value') == '2'
    os.unlink('ReReadingConfigParser.TESTFILE')

    print 'ok'
    
if __name__ == '__main__':
    test()



#!/usr/bin/env python
# encoding: utf-8

"""
checksumming.py - various checksum functions.

Created by Maximillian Dornseif on 2006-11-05.

This file can be used under an BSD License.
"""

__revision__ = "$Revision: 3812 $"

import doctest
import unittest

# The original code came from QBasic code I written from the EAN-Spec in 1986 or so.
# The Python code was inspired by bookland.py and was extended to "longer than EAN"
# by checking the javascript at 
# http://www.gs1.org/productssolutions/barcodes/support/check_digit_calculator.html


def ean_digit(arg):
    """Calculate UPCA/EAN13/NVE checksum for any given string consiting of an arbitary number of digits.
    
    >>> ean_digit('400599871650')
    '2'
    >>> ean_digit('34005998000000027')
    '5'
    """
    
    factor = 3
    summe = 0
    for index in range(len(arg)-1, -1, -1):
        summe += int(arg[index]) * factor
        factor = 4 - factor
    return str((10 - (summe % 10)) % 10)
    

def _ean_digit2(arg):
    """Alternate implementation of EAN check digit calculation used for sanity checks."""
    
    if len(arg) % 2 == 1:
        weight = [3, 1] * ((len(arg) * 2) + 1)
    else:
        weight = [1, 3] * ((len(arg) * 2) + 1)
    magic = 10
    summe = 0
    for i in range(len(arg)):         # checksum based on first 12 digits.
        summe = summe + int(arg[i]) * weight[i]
    ret = (magic - (summe % magic)) % magic
    if ret < 0 or ret >= magic:
        raise RuntimeError("EAN checkDigit: something wrong.")
    return str(ret)
    

def verify_ean(arg):
    """Check if a given strin ends withh a valid check digit.
    
    >>> verify_ean('4005998000007')
    True
    >>> verify_ean('4005998000000')
    False
    >>> verify_ean('foobar')
    False
    """
    
    arg = str(arg)
    if not arg.isdigit():
        return False
    check = arg[-1]
    if ean_digit(arg[:-1]) != check:
        return False
    return True
    

def dpd_digit(arg):
    """Calculates the Checksum for DPD Packets.
    
    See http://static.23.nu/md/Pictures/BIC3_DPDPaketscheinspez_Neu_D_v101.pdf page 22 for an explanation.
    
    Exaple:
    >>> dpd_digit('400599871650')
    'E'
    >>> dpd_digit('007110601632532948375179276')
    'A'
    """

    # to calculate the code values - position equals code value
    _chartable = list("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ")

    mod = 36
    cdigit = mod
    for char in arg:
        cdigit = cdigit + _chartable.index(char.upper())
        if cdigit > mod:
            cdigit = cdigit - mod
        cdigit = 2 * cdigit
        if cdigit > mod:
            cdigit = cdigit - mod - 1
    cdigit = mod + 1 - cdigit
    if cdigit == mod:
        cdigit = 0
    return _chartable[cdigit]

#f = [tuple(),tuple(),tuple(),tuple(),tuple(),tuple(),tuple(),tuple(),tuple(),tuple()]
#f[0] = ( 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 )
#f[1] = ( 1, 5, 7, 6, 2, 8, 3, 0, 9, 4 );
#i=2;
#j=0;
#while(i < 8):
#    f[i] = [tuple(),tuple(),tuple(),tuple(),tuple(),tuple(),tuple(),tuple(),tuple(),tuple()]
#    while(j < 10):
#        f[i][j] = f[i-1][f[1][j]]
#        j += 1
#    j = 0;
#    i += 1
#
#print f


def verhoeff_digit(arg):
    """
    Implemention of Verhoeff's Dihedral Check Digit based on code from Nick Galbreath
    """
    
    # dihedral addition matrix A + B = a[A][B]
    _amatrix = ((0, 1, 2, 3, 4, 5, 6, 7, 8, 9),
                (1, 2, 3, 4, 0, 6, 7, 8, 9, 5),
                (2, 3, 4, 0, 1, 7, 8, 9, 5, 6),
                (3, 4, 0, 1, 2, 8, 9, 5, 6, 7),
                (4, 0, 1, 2, 3, 9, 5, 6, 7, 8),
                (5, 9, 8, 7, 6, 0, 4, 3, 2, 1),
                (6, 5, 9, 8, 7, 1, 0, 4, 3, 2),
                (7, 6, 5, 9, 8, 2, 1, 0, 4, 3),
                (8, 7, 6, 5, 9, 3, 2, 1, 0, 4),
                (9, 8, 7, 6, 5, 4, 3, 2, 1, 0))
          
    # dihedral inverse map, A + inverse[A] = 0
    _inverse = (0, 4, 3, 2, 1, 5, 6, 7, 8, 9)
    # permutation weighting matrix P[position][value]
    _pmatrix = ((0, 1, 2, 3, 4, 5, 6, 7, 8, 9),
                (1, 5, 7, 6, 2, 8, 3, 0, 9, 4),
                (5, 8, 0, 3, 7, 9, 6, 1, 4, 2),
                (8, 9, 1, 6, 0, 4, 3, 5, 2, 7),
                (9, 4, 5, 3, 1, 2, 6, 8, 7, 0),
                (4, 2, 8, 6, 5, 7, 3, 9, 0, 1),
                (2, 7, 9, 3, 8, 0, 6, 4, 1, 5),
                (7, 0, 4, 6, 9, 1, 3, 2, 5, 8))
    
    check = 0 # initialize check at 0
    digit = 0
    i = 0
    for digit in reversed(arg):
        digit = ord(digit) - 48
        check = _amatrix[check][_pmatrix[(i+1) % 8][digit]] # not quite the same...
        i += 1
    return chr(_inverse[check]+48)


# test cases


class _VerhoeffTests(unittest.TestCase):
    
    def test_checkdigit(self):
        self.assertEqual(verhoeff_digit('123456654321'), '9')
        # self.assertEqual(verhoeff_digit('5743839105748193475681981039847561718657489228374'), '3')
        # self.assertEqual(verhoeff_digit('10003729'), '9')
        # self.assertEqual(verhoeff_digit('1125'), '8')
        # self.assertEqual(verhoeff_digit('16412'), '5')
        # self.assertEqual(verhoeff_digit('142857'), '0')
        # self.assertEqual(verhoeff_digit('0000168'), '6')
        # self.assertEqual(verhoeff_digit('04052'), '6')
        # self.assertEqual(verhoeff_digit('1'), '9')
        
        # results from http://www.augustana.ab.ca/~mohrj/algorithms/checkdigit.html and Algorithm::Verhoeff
        self.assertEqual(verhoeff_digit('1'), '5')
        self.assertEqual(verhoeff_digit('11'), '3')
        # self.assertEqual(verhoeff_digit('0000168'), '6')
        # self.assertEqual(verhoeff_digit('04052'), '6')
        # self.assertEqual(verhoeff_digit('142857'), '0')
        # self.assertEqual(verhoeff_digit('16412'), '8')
        # self.assertEqual(verhoeff_digit('1125'), '2')
        self.assertEqual(verhoeff_digit('5743839105748193475681981039847561718657489228374'), '3')
        self.assertEqual(verhoeff_digit('123456654321'), '9')
        self.assertEqual(verhoeff_digit('10003729'), '1')
        self.assertEqual(verhoeff_digit('505'), '3')
        self.assertEqual(verhoeff_digit('050'), '3')
        self.assertEqual(verhoeff_digit('161'), '8')
        self.assertEqual(verhoeff_digit('616'), '8')
        self.assertEqual(verhoeff_digit('272'), '5')
        self.assertEqual(verhoeff_digit('727'), '5')
        self.assertEqual(verhoeff_digit('494'), '1')
        self.assertEqual(verhoeff_digit('949'), '1')
        self.assertEqual(verhoeff_digit('383'), '4')
        self.assertEqual(verhoeff_digit('838'), '9')
        self.assertEqual(verhoeff_digit('505505'), '2')
        self.assertEqual(verhoeff_digit('050050'), '4')
    

class _EanTests(unittest.TestCase):
    """Simple Tests for EAN checkdigit calculation."""
    
    def test_ean_digit1(self):
        """Test known EANs and their checksumms."""
        self.assertEqual(ean_digit(''), '0')
        self.assertEqual(ean_digit('2'), '4')
        self.assertEqual(ean_digit('22'), '2')
        self.assertEqual(ean_digit('222'), '6')
        self.assertEqual(ean_digit('1234567'), '0')
        self.assertEqual(ean_digit('12345678901'), '2')
        self.assertEqual(ean_digit('062235652032'), '4')
        self.assertEqual(ean_digit('978014200057'), '1')
        self.assertEqual(ean_digit('400599871650'), '2')
        self.assertEqual(ean_digit('400599899379'), '8')
        self.assertEqual(ean_digit('1234567890123'), '1')
        self.assertEqual(ean_digit('0000001234567'), '0')
        self.assertEqual(ean_digit('0012345678901'), '2')
        self.assertEqual(ean_digit('0123456789012'), '8')
        self.assertEqual(ean_digit('00000000000000000'), '0')
        self.assertEqual(ean_digit('00000000000000001'), '7')
        self.assertEqual(ean_digit('00000000000000010'), '9')
        self.assertEqual(ean_digit('00000000000000100'), '7')
        self.assertEqual(ean_digit('12345678901234567'), '5')
        self.assertEqual(ean_digit('22222222222222222'), '0')
        self.assertEqual(ean_digit('99999999999999999'), '5')
        self.assertEqual(ean_digit('34005998000000026'), '8')
        self.assertEqual(ean_digit('34005998000000027'), '5')
        self.assertEqual(ean_digit('34005998000000028'), '2')
        self.assertEqual(ean_digit('99999999999999999'), '5')
        self.assertEqual(ean_digit('999999999999999999'), '6')
        self.assertEqual(ean_digit('9999999999999999999'), '9')
        self.assertEqual(ean_digit('99999999999999999999'), '0')
        self.assertEqual(ean_digit('999999999999999999999'), '3')
        self.assertEqual(ean_digit('9999999999999999999999'), '4')
        self.assertEqual(ean_digit('99999999999999999999999'), '7')
        self.assertEqual(ean_digit('999999999999999999999999'), '8')
        self.assertEqual(ean_digit('9999999999999999999999999'), '1')
        self.assertEqual(ean_digit('99999999999999999999999999'), '2')
        self.assertEqual(ean_digit('999999999999999999999999999'), '5')
        self.assertEqual(ean_digit('9999999999999999999999999999'), '6')
        self.assertEqual(ean_digit('99999999999999999999999999999'), '9')
        self.assertEqual(ean_digit('999999999999999999999999999999'), '0')
        self.assertEqual(ean_digit('9999999999999999999999999999999'), '3')
        self.assertEqual(ean_digit('99999999999999999999999999999999'), '4')
        self.assertEqual(ean_digit('999999999999999999999999999999999'), '7')
        self.assertEqual(ean_digit('99999999999999999999999999999999999'), '1')
        self.assertEqual(ean_digit('9999999999999999999999999999999999999999'), '0')
        self.assertEqual(ean_digit('999999999999999999999999999999999999999999999'), '1')
        self.assertEqual(ean_digit('99999999999999999999999999999999999999999999999999'), '0')
        self.assertEqual(ean_digit('9999999999999999999999999999999999999999999999999999999'), '1')
        self.assertEqual(ean_digit('999999999999999999999999999999999999999999999999999999999999'), '0')
        self.assertEqual(ean_digit('9999999999999999999999999999999999999999999999999999999999999'), '3')
        self.assertEqual(ean_digit('99999999999999999999999999999999999999999999999999999999999999'), '4')

    def test_ean_digit2(self):
        """Test the backup EAN calculation function."""
        self.assertEqual(_ean_digit2(''), '0')
        self.assertEqual(_ean_digit2('1234567'), '0')
        self.assertEqual(_ean_digit2('12345678901'), '2')
        self.assertEqual(_ean_digit2('062235652032'), '4')
        self.assertEqual(_ean_digit2('978014200057'), '1')
        self.assertEqual(_ean_digit2('400599871650'), '2')
        self.assertEqual(_ean_digit2('400599899379'), '8')
        self.assertEqual(_ean_digit2('1234567890123'), '1')
        self.assertEqual(_ean_digit2('0000001234567'), '0')
        self.assertEqual(_ean_digit2('0012345678901'), '2')
        self.assertEqual(_ean_digit2('0123456789012'), '8')
        self.assertEqual(_ean_digit2('00000000000000000'), '0')
        self.assertEqual(_ean_digit2('00000000000000001'), '7')
        self.assertEqual(_ean_digit2('00000000000000010'), '9')
        self.assertEqual(_ean_digit2('00000000000000100'), '7')
        self.assertEqual(_ean_digit2('12345678901234567'), '5')
        self.assertEqual(_ean_digit2('22222222222222222'), '0')
        self.assertEqual(_ean_digit2('99999999999999999'), '5')
        self.assertEqual(_ean_digit2('34005998000000026'), '8')
        self.assertEqual(_ean_digit2('34005998000000027'), '5')
        self.assertEqual(_ean_digit2('34005998000000028'), '2')
    

class _DPDTests(unittest.TestCase):
    
    def test_checkdigit(self):
        self.assertEqual(dpd_digit(''), '1')
        self.assertEqual(dpd_digit('2'), 'X')
        self.assertEqual(dpd_digit('22'), 'P')
        self.assertEqual(dpd_digit('01632532948375'), '2')
        self.assertEqual(dpd_digit('007110601632532948375179276'), 'A')
        self.assertEqual(dpd_digit('007110601632532948375101276'), 'O')
        self.assertEqual(dpd_digit('007110601631234567890136276'), '0')
        self.assertEqual(dpd_digit('09445008454226'), 'K')
        self.assertEqual(dpd_digit('09445008454227'), 'I')
        self.assertEqual(dpd_digit('00711060163123456789013627699999999999999999999999999999'), 'O')
        self.assertEqual(dpd_digit('0071106016312345678901362769999999999999999999999999999999'), '5')
        
    
if __name__ == '__main__':
    doctest.testmod()
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
Copyright (c) 2007 HUDORA GmbH. BSD Licensed.
"""

from types import StringType

__revision__ = "$Revision: 3687 $"


def deUTF8(data):
    """This is meant to help with utf-8 data appearing where unicode should apperar."""
    # In particular my DB is returning the wrong thing.
    if type(data) == StringType:
        return data.decode('utf-8')
    return data



from setuptools import setup, find_packages

setup(name='huTools',
      maintainer='Maximillian Dornseif',
      maintainer_email='md@hudora.de',
      url='https://cybernetics.hudora.biz/projects/wiki/huTools/',
      version='0.34a',
      description='Various tiny tools and toys to make Python coding less work more fun.',
      long_description='''huTools is a collection of many totally non earth shattering modules:
      Various tiny tools and toys to make Python coding less work more fun. 
      See https://cybernetics.hudora.biz/projects/wiki/huTools/ for further Information.''',
      license='BSD',
      classifiers=['Intended Audience :: Developers',
                   'Programming Language :: Python'],
      packages = find_packages(),
      zip_safe = False,
)



#!/usr/bin/env python
# encoding: utf-8
"""
printing.py simple minded toolkit for printing on CUPS.

Created by Maximillian Dornseif on 2006-11-19. BSD Licensed.
"""

import os

__revision__ = "$Revision: 3687 $"


def print_file(filename, jobname=None, printer=None, copies=1):
    """Print a file."""
    args = [] # '-n %d' % copies]
    if printer:
        args.append('-P %r' % str(printer))
    # we create several separate print job
    for i in range(copies):
        os.system('/usr/local/bin/lpr %s %r' % (' '.join(args), filename))
    

def print_data(data, jobname=None, printer=None, copies=1, printserver='printserver.local.hudora.biz'):
    """Print a file."""
    args = [] # ['-n %d' % copies]
    if printer:
        args.append('-P %r' % str(printer))
    #if printserver:
    #    args.append('-H %r' % str(printserver))
    #if jobname:
    #    args.append('-J %r' % jobname.replace("'\";./ ", "_"))
    os.system('echo /usr/local/bin/lpr %s >> /tmp/huTools_lplog' % (' '.join(args), ))
    for i in range(copies):
        fd = os.popen('/usr/local/bin/lpr %s >> /tmp/huTools_lplog' % (' '.join(args), ), 'w')
        fd.write(data)
        fd.close()



#!/usr/bin/env python
# encoding: utf-8
"""
fs.py filesystem-related functions

Created by Maximillian Dornseif on 2008-08-16.
Copyright (c) 2008 HUDORA. Consider it BSD Licensed.
"""

import os


def makedirhier(dirnmame):
    """Created 'dirname' if needes and all intermediate directories."""
    
    if not os.path.exists(dirnmame):
        os.makedirs(dirnmame)
